;; mm-util.el
(cond ((condition-case nil
	   (let ((char (make-char 'japanese-jisx0208 36 34)))
	     (eq char (decode-char 'ucs char))) ;; Emacs 23.
	 (error nil))
       (defalias 'mm-ucs-to-char 'identity))
      ((fboundp 'decode-char)
       (defun mm-ucs-to-char (codepoint)
	 "Convert Unicode codepoint to character."
	 (or (decode-char 'ucs codepoint) ?#)))
      ((and (fboundp 'unicode-to-char) ;; XEmacs 21.5.
	    (subrp (symbol-function 'unicode-to-char)))
       (if (featurep 'mule)
	   (defalias 'mm-ucs-to-char 'unicode-to-char)
	 (defun mm-ucs-to-char (codepoint)
	   "Convert Unicode codepoint to character."
	   (or (unicode-to-char codepoint) ?#))))
      ((featurep 'mule)
       (defun mm-ucs-to-char (codepoint)
	 "Convert Unicode codepoint to character."
	 (if (fboundp 'ucs-to-char) ;; Mule-UCS is loaded.
	     (progn
	       (defalias 'mm-ucs-to-char
		 (lambda (codepoint)
		   "Convert Unicode codepoint to character."
		   (condition-case nil
		       (or (ucs-to-char codepoint) ?#)
		     (error ?#))))
	       (mm-ucs-to-char codepoint))
	   (condition-case nil
	       (or (int-to-char codepoint) ?#)
	     (error ?#)))))
      (t
       (defun mm-ucs-to-char (codepoint)
	 "Convert Unicode codepoint to character."
	 (condition-case nil
	     (or (int-to-char codepoint) ?#)
	   (error ?#)))))

;; mm-url.el
(require 'mm-url)
(defun mm-url-decode-entities ()
  "Decode all HTML entities."
  (goto-char (point-min))
  (while (re-search-forward "&\\(#[0-9]+\\|[a-z]+[0-9]*\\);" nil t)
    (let ((elem (if (eq (aref (match-string 1) 0) ?\#)
			(let ((c (mm-ucs-to-char
				  (string-to-number
				   (substring (match-string 1) 1)))))
			  (if (mm-char-or-char-int-p c) c ?#))
		      (or (cdr (assq (intern (match-string 1))
				     mm-url-html-entities))
			  ?#))))
      (unless (stringp elem)
	(setq elem (char-to-string elem)))
      (replace-match elem t t))))
